import $ from './util/dom-core.js';
import {getParentByClassname, getParentNodeByClass, isEmptyElement, trim} from './util/util.js';
import QuestionTools from './util/question';
import QuestionPrevTools from './util/questionPrev';
import JQuery from 'jquery';
let yzPage = function (manage, $page) {
  this.manage = manage;
  this.editor = manage.editor;
  this.pageIndex = this.manage.pageIndex;
  this.id = ('lsiten_editor_page_'+Math.random()).replace('.','_') + this.pageIndex;
  this.$textElem = $('<div class="js-page-content" id = "' +this.id+ '"></div>');
  this.$colums = [];
  this.paragraphIndex = 0;
  // 已经初始化后的内容
  this.$page = $page;
  this.lineHeight = 25;
  this._init();
  manage.$el.append(this.$textElem);
}

yzPage.prototype = {
  constructor: yzPage,
  _init: function () {
    this._initStyles();
    this._initColumn();
    // this._initPositionPlaceholder();
    // this._initPAndNPlaceholder();
    this.questionTools = new QuestionTools();
    this.questionPrevTools = new QuestionPrevTools();
  },
  /**
   * function 初始化样式
   */
  _initStyles: function () {
    this.$textElem.css('position', 'relative')
                  .css('outline', 'none')
                  .css('word-wrap', 'break-word')
                  .css('font-size', '14px')
                  .css('-webkit-line-break', 'after-white-space')
                  .css('height', this._pagesHeight[this.editor.pageSize])
  },
  /**
   * function 初始化栏
   */
  _initColumn: function () {
    let colums = this.editor.columnNumber;
    let height = this._pagesHeight[this.editor.pageSize];
    let widthColumn = 100 / colums + '%';
    // 初始化栏数据
    let $columnData = [];
    if (this.$page) {
      $columnData = this.$page.find('.js-column');
    }
    for (let i = 0; i < colums; i++) {
      let $columItem = null;
      if ($columnData[i]) {
        $columItem = $($columnData[i]);
      } else {
        $columItem = $('<div class="lsiten-colum-' + this.pageIndex + '-' + i + ' js-column"></div>');
        $columItem.attr('data-column-i', i);
        $columItem.attr('data-column-page', this.pageIndex);
        // $columItem.attr('contenteditable', true);
        $columItem.css('height', height);
        $columItem.css('position', 'relative');
        $columItem.css('overflow', 'hidden');
        $columItem.css('float', 'left');
        $columItem.css('width', widthColumn);
      }
      this.$colums.push($columItem);
      this.$textElem.append($columItem);
    }
    this.currentColumn = 0;
  },
  /**
   * function 添加表头
   */
  addHeader: function () {
    this.editor.cmd.do('answerHeader', this);
  },

  /**
   * function 添加段落
   */
  addParagraph: function () {
    let $column = this.$colums[this.currentColumn];
    let $paragraph = $('<div class="js-paragraph-view"></div>');
    $paragraph.css('margin', 0)
              .css('padding', '0 25px')
              .css('outline', 'none')
              .css('box-sizing', 'border-box')
              .css('white-space', 'pre-wrap')
              .css('line-height', '1.75')
              .css('text-align', 'left');
    let index = this.pageIndex + '-' + this.paragraphIndex++;
    $paragraph.attr('data-index', index);
    let $currentParagraph = this.manage.editorPosition.currentParagraph;
    if (!$currentParagraph && !$column[0].childNodes.length) {
      $column.append($paragraph);
    } else {
      if (this.manage.editorPosition.currentParagraph !== this.$el) {
        $column.append($paragraph);
      } else {
        $paragraph.insertAfter($currentParagraph);
      }
    }
    return $paragraph;
  },
  // 更新头部
  updateHeader: function () {
  },

  _placeholderTemplate: function () {
    let $postionDom = $('<div></div>');
    $postionDom.css('background-color', '#000');
    $postionDom.css('position', 'absolute');
    $postionDom.attr('contenteditable', false);
    $postionDom.html('&#8203;');
    $postionDom.css('width', '25px');
    $postionDom.css('height', '16px');
    return $postionDom;
  },
  // 生成正反面定位符
  _initPAndNPlaceholder: function () {
  },
  // 初始化占位符
  _initPositionPlaceholder: function () {
    let $columns = this.$colums;
    let clength = $columns.length;
    let cwidth = this._pagesWidth[this.editor.pageSize] / clength;
    let lastIndex = clength - 1;
    for (let i = 0; i < clength; i++) {
      let $template = this._placeholderTemplate();
      let $templateb = this._placeholderTemplate();
      let left = i * cwidth - 25;
      if (i > 0) {
        left += 15
        $template.css('left', left + 'px').css('top', '-16px');
        this.$textElem.append($template);
      }
      
      $templateb.css('left', left + 'px').css('bottom', '-16px');
      this.$textElem.append($templateb);

      if (i === lastIndex) {
        let $template1 = this._placeholderTemplate();
        let $template1b = this._placeholderTemplate();
        $template1.css('right', '-25px').css('top', '-16px');
        $template1b.css('right', '-25px').css('bottom', '-16px');
        this.$textElem.append($template1);
        this.$textElem.append($template1b);
      }
    }
  },
  // 更新跨栏符
  _updateCrossColumn: function ($column) {
  },
  
  /** 
   * @param {DomElement} $column 栏dom
  */
  _checkColumnOut($column) {
    let $paragraphs = $column.find('.js-paragraph-view');
      let cSize = $column.getSizeData();
      let i = $paragraphs.length - 1;
      if(!i || i < 0) {
        return false;
      }
      let $moves = [];
      while (i >=0 ) {
        let $paragraph = $($paragraphs[i]);
        let pSize = $paragraph.getSizeData();
        if (pSize.top >= cSize.bottom) {
          $moves.push($paragraph);
        } else if (pSize.bottom >= cSize.bottom) {
          this._splitParagraph($moves, $paragraph, cSize, $column);
        }
        i--;
      }
      if ($moves.length > 0) {
        let nextColumn = this._getNextColumn($column, true);
        if (nextColumn.column) {
          let $firsParagraph = this._getFirstPargraph(nextColumn.column);
          if ($firsParagraph) {
            $moves.forEach(item => {
              this._insertParagraph(item, $firsParagraph);
            })
          } else {
            let moveIndex = $moves.length - 1;
            while (moveIndex >=0 ) {
              nextColumn.column.append($moves[moveIndex]);
              moveIndex--;
            }
          }
          nextColumn.page._checkColumnOut(nextColumn.column);
        }
      }
  },
  /**
   * function 插入段落
  * @param {DomElement}  $insertParagraph 需要插入的段落
   * @param {DomElement}  $firsParagraph 插入该段之前
   */
  _insertParagraph: function ($insertParagraph, $firsParagraph) {
    let insertClass = $insertParagraph.attr('class');
    let firstClass = $firsParagraph.attr('class');
    if (insertClass && firstClass && insertClass.indexOf('js-split-paragraph') >-1 && firstClass.indexOf('js-split-paragraph') >-1) {
      let insertLast = $insertParagraph[0].lastChild;
      let firstFirst = $firsParagraph[0].firstChild;
      this._mergeSplitParagraph(insertLast, firstFirst, $insertParagraph);
    } else {
      $insertParagraph.insertBefore($firsParagraph);
    }
  },
  /**
   * function 插入内容在该dom之前
   * @param {DomElement}
   */
  _insertDomBeforeByFirst: function ($paragraph, first) {
    $paragraph.childNodes().forEach(child => {
      $(child).insertBefore($(first));
    })
  },
  /**
   * function 合并拆分过的段落
   * @param {node} 上一段最后一个dom
   * @param {node} 下一段第一个dom
   * @param {DomElement} 需要插入的段落
   */
  _mergeSplitParagraph: function (insertLast, firstFirst, $Allparagraph) {
    if (!insertLast) {
      return false;
    }
    if (insertLast.className && insertLast.className.indexOf('js-lsiten-line') > -1) {
      // 普通行处理
      this._insertDomBeforeByFirst($Allparagraph, firstFirst);
    } else if (insertLast.className && insertLast.className.indexOf('js-lsiten-border') > -1) {
      // 框处理
      if (firstFirst.className && firstFirst.className.indexOf('js-lsiten-border') > -1) {
        // 如果下一段第一个dom是框
        let borderId = insertLast.getAttribute('data-id');
        let nextBorderId = firstFirst.getAttribute('data-id');
        if (borderId === nextBorderId) {
          // 如果两个框是同一个框
          let insertLastQuestion = insertLast.firstChild.lastChild;
          let firstFirstQuestion = firstFirst.firstChild.firstChild;
          this._mergeSplitBorder(insertLastQuestion, firstFirstQuestion, $(insertLast));
          this._insertDomBeforeByFirst($Allparagraph, firstFirst);
        } else {
          // 如果不是同一个框
          this._insertDomBeforeByFirst($Allparagraph, firstFirst);               
        }
      } else {
        // 如果下一段第一个dom不是框
        this._insertDomBeforeByFirst($Allparagraph, firstFirst);
      }
      
    } else {
      let parentNode = insertLast.parentNode;
      $(insertLast).remove();
      this._mergeSplitParagraph(parentNode.lastChild, firstFirst);
    }
  },
  /**
   * function 合并框
   * @param {node} lQuestion 上一段最后一个问题
   * @param {node} fQuestion 下一段第一个问题
   * @param {DomElement} $Allborder 该框所有dom
   */
  _mergeSplitBorder: function (lQuestion, fQuestion, $Allborder) {
    if (!lQuestion) {
      return false;
    }
    if (lQuestion.className && lQuestion.className.indexOf('js-lsiten-line') > -1) {
      // 普通行处理
      this._insertDomBeforeByFirst($Allborder, fQuestion);
      $Allborder.remove();
    } else if (lQuestion.className && lQuestion.className.indexOf('js-lsiten-question') > -1) {
      // 主观题才需要合并
      let type = parseInt(lQuestion.getAttribute('data-type'));
      if (type > 2) {
        if (fQuestion.className && fQuestion.className.indexOf('js-lsiten-question') > -1) {
          let questionId = lQuestion.getAttribute('data-id');
          let nextQuestionId = fQuestion.getAttribute('data-id');
          if (questionId === nextQuestionId) {
            // 不是同一个题
            if (type === 3) {
              // 客观题
              let $lquestion = $(lQuestion);
              let fQuestionFirst = fQuestion.firstChild;
              this._insertDomBeforeByFirst($lquestion, fQuestionFirst);
              $lquestion.remove();
              this._insertDomBeforeByFirst($($Allborder[0].firstChild), fQuestion);
              $Allborder.remove();
            } else if (type === 4) {
              // 作文题
              let questionContent = lQuestion.lastChild;
              let firstContent = fQuestion.firstChild;
              if (questionContent.className && questionContent.className.indexOf('lsiten-title') > -1) {
                if (firstContent.className && firstContent.className.indexOf('lsiten-title') > -1) {
                  let $writtingTitle = $(questionContent);
                  this._insertDomBeforeByFirst($writtingTitle, firstContent.firstChild);
                  $writtingTitle.remove();
                  this._insertDomBeforeByFirst($(lQuestion), firstContent);
                  $(lQuestion).remove();
                  this._insertDomBeforeByFirst($($Allborder[0].firstChild), fQuestion);
                  $Allborder.remove();
                } else {
                  this._insertDomBeforeByFirst($(lQuestion), firstContent);
                  $(lQuestion).remove();
                  this._insertDomBeforeByFirst($($Allborder[0].firstChild), fQuestion);
                  $Allborder.remove();
                }
              } else if (questionContent.className && questionContent.className.indexOf('lsiten-border-box') > -1) {
                if (firstContent.className && firstContent.className.indexOf('lsiten-border-box') > -1) {
                  let $writtingRows = $(questionContent);
                  this._insertDomBeforeByFirst($writtingRows, firstContent.firstChild);
                  $writtingRows.remove();
                  this._insertDomBeforeByFirst($(lQuestion), firstContent);
                  $(lQuestion).remove();
                  this._insertDomBeforeByFirst($($Allborder[0].firstChild), fQuestion);
                  $Allborder.remove();
                } else {
                  this._insertDomBeforeByFirst($(lQuestion), firstContent);
                  $(lQuestion).remove();
                  this._insertDomBeforeByFirst($($Allborder[0].firstChild), fQuestion);
                  $Allborder.remove();
                }
              } else {
                questionContent && questionContent.parentNode.removeChild(questionContent);
              }
            }
          } else {
            // 不是同一个题
            this._insertDomBeforeByFirst($($Allborder[0].firstChild), fQuestion);
            $Allborder.remove();
          }
        } else {
          // 下一段第一个不是一个问题
          this._insertDomBeforeByFirst($($Allborder[0].firstChild), fQuestion);
          $Allborder.remove();
        }
      } else {
        // 客观题处理
        this._insertDomBeforeByFirst($($Allborder[0].firstChild), fQuestion);
        $Allborder.remove();
      }
    } else {
      let parentNode = lQuestion.parentNode;
      $(lQuestion).remove();
      this._mergeSplitBorder(parentNode.lastChild, fQuestion, $Allborder);
    }
  },
   /**
   * function 检测内容是否需要回退
   */
  _checkContentBack: function () {
    let $columns = this.$colums;
    // 备用
    let maxHeight = parseInt(this._pagesHeight[this.editor.pageSize]);
    $columns.forEach($column => {
      this._checkColumnBack($column);   
    });
  },
  /**
   * function 检测该段落是否为答题卡头
   * @param {DomElement} 需要检测的段落
   */
  isHeadParagraph: function ($paragraph) {
    let firstChild = $paragraph[0].firstChild;
    return firstChild && firstChild.className && firstChild.className.indexOf('js-answer-header') > -1;
  },
  /**
   * function 检测栏是否需要回退
   * @param {DomElement} $column 需要检测的栏
   */
  _checkColumnBack: function ($column) {
    let $paragraphs = $column.find('.js-paragraph-view');
    let cSize = $column.getSizeData();
    let i = $paragraphs.length - 1;
    if(!i || i < 0) {
      return false;
    }
    let $moves = [];
    let $lastParagraph = $($paragraphs[i]);
    let lastSize = $lastParagraph.getSizeData();
    let diff = cSize.bottom - lastSize.bottom;
    if (diff > this.lineHeight) {
      let nextColumn = this._getNextColumn($column, false);
      // 如果有下一页，进行检测
      if (nextColumn.column) {
        let $nextParagraphs = nextColumn.column.find('.js-paragraph-view');
        while (i >=0 ) {
          let $paragraph = $($nextParagraphs[i]);
          if (!this.isHeadParagraph($paragraph)) {
            let pSize = $paragraph.getSizeData();
            if (pSize.height < diff) {
              $moves.push($paragraph);
            } else {
              this._prevParagraph($moves, $paragraph, diff, $lastParagraph);
            }
          }
          i--;
        }
      }
    }
  },

  /**
   * function 检测段落是否需要上移
   * @param {array} $moves 需要移动的段落数组
   * @param {DomElement} $paragraph  需要检测的段落
   * @param {number} diff  可以上移的空间
   * @param {DomElement} $lastParagraph  该段最后一段
   */
  _prevParagraph: function ($moves, $paragraph, diff, $lastParagraph) {
    let lastClass = $lastParagraph.attr('class');
    let currentClass = $paragraph.attr('class');
    let lastPid = $lastParagraph.attr('data-index');
    let currentPid = $paragraph.attr('data-index');
    if (lastClass && lastClass.indexOf('js-split-paragraph') > -1) {
      if (currentClass && currentClass.indexOf('js-split-paragraph') > -1 && lastPid === currentPid) {
        // 如果两个段落需要合并
        let $prevParagraph = $paragraph.clone(false);
        $paragraph.childNodes().forEach(item => {
          if (item.nodeType === 3) {
            $paragraph[0].removeChild(item);
          } else {
            let $item = $(item);
            let itemClass = $item.attr('class');
            if (itemClass && itemClass.indexOf('js-lsiten-line') > -1) {
              // 普通行
              let height = $item.getSizeData().height;
              if (height < diff) {
                $prevParagraph.append($item);
                diff -= height;
              }
            } else if (itemClass && itemClass.indexOf('js-lsiten-border') > -1) {
              // 框处理
              this._prevBorder($prevParagraph, diff, $item, $lastParagraph);
            } else {
              $item.remove();
            }
          }
        })
        if (!isEmptyElement($prevParagraph[0])) {
          $moves.push($prevParagraph);
        }
      } else {

      }
    } else {

    }
  },

  /**
   * function 处理需要上移的框
   * @param {DomElement} $prevParagraph 前移段落暂存
   * @param {number} diff 可移空间
   * @param {DomElement} $border 需要处理的框
   * @param {DomElement} $lastParagraph 最后一段dom
   */
  _prevBorder: function ($prevParagraph, diff, $border, $lastParagraph) {
    let lastCentent = $lastParagraph[0].lastChild;
    if (lastCentent.className && lastCentent.className.indexOf('js-lsiten-border') > -1) {
      let prevBid = $border.attr('data-id');
      let bid = $(lastCentent).attr('data-id');
      if (prevBid === bid) {
        let $prevBorder = $border.clone(false);
        this._prevQuestions($prevBorder, diff, $border, $(lastCentent));
      } else {
        let borderSize= $border.getSizeData();
        if (borderSize.height < diff) {
          $prevParagraph.append($border);
          diff -= borderSize.height;          
        }
      }
    } else {
      let borderSize= $border.getSizeData();
      if (borderSize.height < diff) {
        $prevParagraph.append($border);
        diff -= borderSize.height;
      }
    }
  },
  /**
   * function 前移问题
   * @param {DomElement} $prevBorder 前移框暂存
   * @param {number} diff 前移段落暂存
   * @param {DomElement} $border 前移段落暂存
   * @param {DomElement} $lastBorder 前段落最后一个框
   * 
   */
  _prevQuestions: function ($prevBorder, diff, $border, $lastBorder) {
    let lastQuestion = $lastBorder[0].firstChild.lastChild;
    let $borderContent = $($border[0].firstChild);
    $borderContent.childNodes().forEach(item => {
      if (item.className) {
        if (item.className.indexOf('js-lsiten-question') > -1) {
          let qustionType = parseInt(item.getAttribute('data-type'));
          let questionId = item.getAttribute('data-id');
          if (lastQuestion.className && lastQuestion.className.indexOf('js-lsiten-question') > -1) {

          } else {
            if (qustionType === 3) {
              let $prevSubject = $(item).clone(false);
              this._prevSubjectQuestion($prevSubject, diff, $(item));
              if (!isEmptyElement($prevSubject)) {
                $prevBorder[0].firstChild.appendChild($prevSubject[0]);
              }
            } else if (qustionType === 4) {
              let $prevWritting = $(item).clone(false);
              this._prevWrittingQuestion($prevWritting, diff, $(item));
              if (!isEmptyElement($prevWritting)) {
                $prevBorder[0].firstChild.appendChild($prevWritting[0]);
              }
            } else {
              let height = $(item).getSizeData();
              if (height < diff) {
                $prevBorder[0].firstChild.appendChild(item);
                diff -= height;
              }
            }
          }
        } else if (item.className.indexOf('js-lsiten-line') > -1) {
          let height = $(item).getSizeData();
          if (height < diff) {
            $prevBorder[0].firstChild.appendChild(item);
            diff -= height;
          }
        } else {
          $(item).remove();
        }
      } else {
        item && item.parentNode.removeChild(item);
      }
    })
  },
  /**
   * function 前移主观题框
   * @param {DomElement} $prevSubject  前移框暂存
   * @param {number} diff  可移动空间
   * @param {DomElement} $subject  需要前移客观题处理
   */
  _prevSubjectQuestion: function ($prevSubject, diff, $subject) {
    $subject.childNodes().forEach(item => {
      let height = $(item).getSizeData();
      if (height < diff) {
        $prevSubject[0].appendChild(item);
        diff -= height;
      }
    })
  },
  /**
   * function 前移主观题框
   * @param {DomElement} $prevSubject  前移框暂存
   * @param {number} diff  可移动空间
   * @param {DomElement} $subject  需要前移客观题处理
   */
  _prevWrittingQuestion: function ($prevWritting, diff, $Writting) {
    $Writting.childNodes().forEach(item => {
      if (item.className) {
        if (item.className.indexOf('lsiten-title') > -1) {
  
        } else if (item.className.indexOf('lsiten-border-box') > -1) {
  
        }
      }
    })
  },
  /**
   * function 检测内容是否超出
   */
  _checkContentOut: function () {
    let $columns = this.$colums;
    // 备用
    let maxHeight = parseInt(this._pagesHeight[this.editor.pageSize]);
    $columns.forEach($column => {
      this._checkColumnOut($column);   
    });
  },
  /**
   * function 获取第一段 
   */
  _getFirstPargraph: function (column) {
    let $paragraphs = column.find('.js-paragraph-view');
    let firstParagraph = $paragraphs[0];
    let firstChild = firstParagraph.firstChild;
    if (firstChild && firstChild.className && firstChild.className.indexOf('js-answer-header') > -1) {
      firstParagraph = $paragraphs[1];
    }
    return firstParagraph ? $(firstParagraph): false;
  },
  /**
   * function 插入dom
   * @param {DomElement}  $nextDom 下一栏暂存
   * @param {DomElement}  $dom 需要插入的dom
   */
  insertDom($nextDom, $dom) {
    let firstChild = $nextDom[0].firstChild;
    if (firstChild) {
      $dom.insertBefore($(firstChild));
    } else {
      $nextDom.append($dom);
    }
  },
  /**
   * function 拆分段落
   * @param $moves {Array} 需要移动的数组
   * @param $paragraph {DomElement} 需要切分的段落
   * @param cSize {Object} 段落尺寸数据
   * @param $column {DomElement} 当前栏
   */
  _splitParagraph: function ($moves, $paragraph, cSize, $column) {
    let $nextParagraph = $paragraph.clone(false);
    let childs = $paragraph[0].childNodes;
    let index = childs.length - 1;
    while (index >= 0) {
      let line = childs[index];
      if (line && line.className) {
        if (line.className.indexOf('js-lsiten-line') > -1) {
          // 普通一行
          let $line = $(line);
          let lineSize = $line.getSizeData();
          if (lineSize.top >= cSize.bottom) {
            this.insertDom($nextParagraph, $line);
          }
        } else if (line.className.indexOf('js-lsiten-border') > -1) {
          // 题框
          this._splitBorder($nextParagraph, $(line), cSize);
        } else {
          $(line).remove();
        }
      } else {
        line && $paragraph[0].removeChild(line);
      }
      index--;
    }
    if ($nextParagraph.childNodes().length > 0) {
      $paragraph.addClass('js-split-paragraph');
      $nextParagraph.addClass('js-split-paragraph');
      $moves.push($nextParagraph);
    }
  },

  /**
   * function 获取下一栏
   * @param {DomElement} $column 当前栏
   * @param {boolean} isAddpage 是否需要新增页
   */
  _getNextColumn: function ($column, isAddPage) {
    let nextColumnIndex = parseInt($column.attr('data-column-i')) + 1;
    let $nextColumn = this.$colums[nextColumnIndex];
    if ($nextColumn) {
      return {
        page: this,
        column: $nextColumn
      };
    } else {
      let page = this._getNextPage(isAddPage);
      if (page) {
        $nextColumn = page.$colums[0];
        return {
          page: page,
          column: $nextColumn
        };
      } else {
        return {
          page: false,
          column: false
        };
      }
    }
  },
  /**
   * function 获取下一页对象
   * @param {Boolean} isAddPage 如果没有是否添加一页
   */
  _getNextPage: function (isAddPage) {
    let nextPageIndex = this.pageIndex + 1;
    let page = this.manage.getNextPageByKey(nextPageIndex, isAddPage)
    return page;
  },
  /**
   * function 拆分框
   * @param $nextParagraph {DomElement} 下一段暂存
   * @param $border {DomElement} 需要切分的框
   * @param cSize {Object} 段落尺寸数据
   */
  _splitBorder: function ($nextParagraph, $border, cSize) {
    let $splitBorder = this.questionTools._splitBorder($border, cSize);
    if ($splitBorder) {
      this.insertDom($nextParagraph, $splitBorder);
    }
  },
  
  _pagesHeight: {
    'A4': '970px',
    'A3': '970px'
  },
  _pagesWidth: {
    'A4': 653,
    'A3': 1447
  }
}

export default yzPage
