import $ from './util/dom-core.js';
import { gennerOneLine } from './util/lineUtil.js';
import Jquery from 'jquery';
import yzPage from './yzPage.js';
import { UA, isFunction, replaceHtmlSymbol, getParentByClassname, trim, throttle, isEmptyElement, getParentNodeByClass } from './util/util.js';

let yzPageManager = function (editor) {
  this.editor = editor;
  this.id = ('lsiten_editor_page_'+Math.random()).replace('.','_');
  this.pageIndex = 0;
  this.currentPage = null;
  this.pages = [];
  // enter键状态
  this._event = null;
  this._checkPageOutStatus = false;  
  this._checkPageBackStatus = false;  

  // 输入法监听 
  this.inputType = false;

  this.$el = $('<div id = " ' + this.id + ' " class="lsiten-pages-view"></div>');

  this.ready = false;
  this.editorPosition = {};
  this._$handleImg = null;
  this._init();
}

yzPageManager.prototype = {
  constructor: yzPageManager,
  _init: function () {
    this._initStyles();
    this._addPage();
    this._initEvents();
  },
  /**
   * function 初始化页面的样式
   */
  _initStyles: function () {
    this.$el.css('width', this._pagesWidth[this.editor.pageSize])
            .css('background', '#ffffff')
            .css('position', 'relative')
            .css('padding', '70px!important')
            .css('box-sizing', 'border-box')
            .css('min-width', '0')
            .css('box-shadow', '0 2px 7px rgba(0,0,0,.1)')
            .css('overflow', 'hidden')
            .css('margin', '0 auto')
            .css('height', this._pagesHeight[this.editor.pageSize]);
  },
  /**
   * function 添加一页
   */
  _addPage: function () {
    this._addBreakPlaceholder();    
    let page = new yzPage(this, '');
    this.pages.push(page);
    this.pageIndex++;
    page.addHeader();
    return page;
  },
  /**
   * function 通过key值获取下一页
   * @param {number} key page的key值
   * @param {Boolean} isAddPage 如果没有是否添加一页
   */
  getNextPageByKey: function (key, isAddPage) {
    let page = this.pages[key];
    if (page) {
      return page;
    } else {
      if (isAddPage) {
        return this._addPage();
      } else {
        return false;
      }
    }
  },
  _addBreakPlaceholder: function () {
    if (this.pageIndex < 1) {
      return '';
    }
    // 添加分页显示效果
    let $breakDiv = $('<div class="page-break js-page-break-' + (this.pageIndex) + '" data-page-id="' + (this.pageIndex) + '"></div>');
    let $topSpan = $('<div class="spanMark topSpan">&#8203;</div>');
    $topSpan.css('display', 'block').css('height', '71.5px');
    // 分页符
    let $middleSpan = $('<div class="spanMark middleSpan lsiten-page-break-span">&#8203;</div>');
    $middleSpan.css('page-break-before', 'always');
    $middleSpan.css('display', 'block').css('height', '10px').css('background-color', '#f6f6f6').css('box-shadow', 'inset 0 7px 7px -5px rgba(0, 0, 0, 0.1), inset 0 -5px 5px -7px rgba(0, 0, 0, 0.1), 7px 0 7px #f6f6f6, -7px 0 7px #f6f6f6');  
    let $bottomSpan = $('<div class="spanMark bottomSpan">&#8203;</div>');
    $bottomSpan.css('height', '71.5px').css('display', 'block');

    // 添加页眉页脚   
    $breakDiv.append($topSpan);
    $breakDiv.append($middleSpan);
    $breakDiv.append($bottomSpan);
    $breakDiv.css('height', '153px');
    this.$el.append($breakDiv);

    let height = this.pages.length * this._pagesBreakHeight[this.editor.pageSize] + parseInt(this._pagesHeight[this.editor.pageSize]);
    this.$el.css('height', height + 'px');
  },

  _checkEveryPageOut: function () {
    if (this._checkPageOutStatus) {
      return false;
    }
    this._checkPageOutStatus = true;
    let pages = this.pages;
    pages.forEach(page => {
      page._checkContentOut();
    });
    this._checkPageOutStatus = false;
  },

  _checkEveryPageBack: function () {
    if (this._checkPageBackStatus) {
      return false;
    }
    this._checkPageBackStatus = true;
    let pages = this.pages;
    pages.forEach(page => {
      page._checkContentBack();
    });
    this._checkPageBackStatus = false;
  },
  /**
   * function 添加事件监听 
   */
  _initEvents: function () {
    this._realtimeSavePosition();
    this._enterEvent();
    this._backEvent();
    this._delEvent();

    this._contentchangeEvent();
  },
   /** 
   * function 内容变化事件监听
  */
  _contentchangeEvent: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('addContent', function (e) {
      _this._checkEveryPageOut();
    })

    Jquery($textElem[0]).on('removeContent', function (e) {
      _this._checkEveryPageBack();
    })
  },
  /** 
   * function del键监听
  */
  _delEvent: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('keydown', function (e) {       
      if (e.keyCode !== 46) {
        // 不是回车键
        return '';
      }
      if (getParentByClassname($(e.target), 'js-answer-header-title')) {
        return '';
      }
      let $currentLine = _this.editorPosition.currentLine;
      if ($currentLine) {
        let selection = _this.editor.selection;
        if (selection.isCartEnd($currentLine)) {
          let nextLine = $currentLine[0].nextSibling;
          if (nextLine && nextLine.className && nextLine.className.indexOf('js-lsiten-line') > -1 && isEmptyElement(nextLine)) {
            $(nextLine).remove();
            Jquery($textElem[0]).trigger('removeContent');
            return false;          
          } else {
            return false;
          }
        }
      } else {
        return false;
      }
    })
  },
  /** 
   * function 退格键监听
  */
  _backEvent () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('keydown', function (e) {       
      if (e.keyCode !== 8) {
        // 不是回车键
        return '';
      }
      if (getParentByClassname($(e.target), 'js-answer-header-title')) {
        return '';
      }
      let $currentLine = _this.editorPosition.currentLine;
      if ($currentLine) {
        if (isEmptyElement($currentLine[0])) {
          let prevLine = $currentLine[0].previousSibling;
          if (prevLine && prevLine.className && prevLine.className.indexOf('js-lsiten-line') > -1) {
            $currentLine.remove();
            prevLine.focus();
            Jquery($textElem[0]).trigger('removeContent');
            return false;            
          } else {
            return false;
          }
        }
      } else {
        return false;
      }
    })
  },
  /**
   * function 监听enter键事件
   */
  _enterEvent: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    Jquery($textElem[0]).on('keydown', function (e) {       
      if (e.keyCode !== 13) {
        // 不是回车键
        return '';
      }
      if (getParentByClassname($(e.target), 'js-answer-header-title')) {
        return '';
      }
      let $currentLine = _this.editorPosition.currentLine;
      if ($currentLine) {
        let $newLine = gennerOneLine();
        $newLine.insertAfter($currentLine);
        $newLine.focus();
      }

      Jquery($textElem[0]).trigger('addContent');
      return false;
    })
  },
  /**
   * function 实时保存当前编辑信息
   */
  _realtimeSavePosition: function () {
    const $textElem = this.$el;
    const editor =this.editor;
    let _this = this;
    function realTimeSave (e) {
      _this._updatePositionInfo(e);
    }

    Jquery($textElem[0]).on('compositionend', function () {       
      _this.inputType = false;
    })
    Jquery($textElem[0]).on('keyup',  realTimeSave);
    Jquery($textElem[0]).on('keydown',  realTimeSave);
    Jquery($textElem[0]).on('mousedown',  e => {
      // mousedown 状态下，鼠标滑动到编辑区域外面，也需要保存选区
      Jquery($textElem[0]).on('mouseleave', '.js-column', realTimeSave);
    })
    Jquery($textElem[0]).on('mouseup', '.js-column', e => {
        realTimeSave(e);
        // 在编辑器区域之内完成点击，取消鼠标滑动到编辑区外面的事件
        Jquery($textElem[0]).off('mouseleave', realTimeSave);
    })
  },
  /**
   * 更新当前编辑信息
   */
  _updatePositionInfo: function (e, $ele) {
    let $target = $ele || $(e.target || e.toElement);
    let $border = getParentByClassname($target, 'js-lsiten-border');
    let $line = getParentByClassname($target, 'js-lsiten-line');
    let $paragraph = getParentByClassname($target, 'js-paragraph-view');
    if (!$paragraph) {
      return false;
    }

    let $currentColumn = getParentByClassname($paragraph, 'js-column');
    if (!$currentColumn) {
      console.log($target, console.log("%c数据错误！", "color:red"));
      return false;
    }
    this.currentPage = this.pages[parseInt($currentColumn.attr('data-column-page'))];
    if (!this.currentPage) {
      console.log($target, $currentColumn, $paragraph, console.log("%c数据错误！", "color:red"));
      return false;
    }
    this.currentPage.currentColumn = parseInt($currentColumn.attr('data-column-i'));

    this.editorPosition = {
      currentParagraph: $paragraph,
      currentColumn: this.currentPage.$colums[this.currentPage.currentColumn],
      currentPage: this.currentPage,
      currentBorder: $border,
      currentLine: $line
    }

    let selection = this.editor.selection;
    if ($line) {
      if (!selection.getRange() ||  !selection.getSelectionContainerElem()) {
        selection.createRangeByElem($line, true);
      } else {
        selection.saveRange();
      }
    } else {
      selection.setEmptyRange();
    }
  },
  _pagesWidth: {
    'A4': '793px',
    'A3': '1587px'
  },
  _pagesHeight: {
    'A4': '1122px',
    'A3': '1122px'
  },
  _pagesBreakHeight: {
    'A4': '1122',
    'A3': '1122'
  }
}
export default yzPageManager
